>From 37bcb06a5ac00dee552acc2dd8522a1296474e1b Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Jan=20Urba=C5=84ski?= <wulczer@wulczer.org>
Date: Sun, 2 Jan 2011 11:19:20 +0100
Subject: [PATCH 05/16] Use palloc in TopMemoryContext instead of malloc.

As discussed, even if the PL needs a permanent memory location, it
should use palloc, not malloc. It also makes error handling easier.
---
 src/pl/plpython/plpython.c |   19 +++++++------------
 1 files changed, 7 insertions(+), 12 deletions(-)

diff --git a/src/pl/plpython/plpython.c b/src/pl/plpython/plpython.c
index f5079a3..c949b7e 100644
--- a/src/pl/plpython/plpython.c
+++ b/src/pl/plpython/plpython.c
@@ -1590,7 +1590,7 @@ PLy_procedure_compile(PLyProcedure *proc, const char *src)
 	 */
 	msrc = PLy_procedure_munge_source(proc->pyname, src);
 	crv = PyRun_String(msrc, Py_file_input, proc->globals, NULL);
-	free(msrc);
+	pfree(msrc);
 
 	if (crv != NULL)
 	{
@@ -1627,7 +1627,7 @@ PLy_procedure_munge_source(const char *name, const char *src)
 	 */
 	mlen = (strlen(src) * 2) + strlen(name) + 16;
 
-	mrc = PLy_malloc(mlen);
+	mrc = palloc(mlen);
 	plen = snprintf(mrc, mlen, "def %s():\n\t", name);
 	Assert(plen >= 0 && plen < mlen);
 
@@ -3629,19 +3629,14 @@ PLy_traceback(int *xlevel)
 
 /* some dumb utility functions */
 static void *
-PLy_malloc(size_t bytes)
+PLy_malloc(Size bytes)
 {
-	void	   *ptr = malloc(bytes);
-
-	if (ptr == NULL)
-		ereport(FATAL,
-				(errcode(ERRCODE_OUT_OF_MEMORY),
-				 errmsg("out of memory")));
-	return ptr;
+	/* We need our allocations to be long-lived, so use TopMemoryContext */
+	return MemoryContextAlloc(TopMemoryContext, bytes);
 }
 
 static void *
-PLy_malloc0(size_t bytes)
+PLy_malloc0(Size bytes)
 {
 	void	   *ptr = PLy_malloc(bytes);
 
@@ -3666,7 +3661,7 @@ PLy_strdup(const char *str)
 static void
 PLy_free(void *ptr)
 {
-	free(ptr);
+	pfree(ptr);
 }
 
 /*
-- 
1.7.2.3

